==> moengagesdk-4.1.07-sources.jar: com/moengage/core/analytics/AnalyticsHelper.java <==
/*
 * Copyright (c) 2014-2020 MoEngage Inc.
 *
 * All rights reserved.
 *
 *  Use of source code or binaries contained within MoEngage SDK is permitted only to enable use of the MoEngage platform by customers of MoEngage.
 *  Modification of source code and inclusion in mobile apps is explicitly allowed provided that all other conditions are met.
 *  Neither the name of MoEngage nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *  Redistribution of source code or binaries is disallowed except with specific prior written permission. Any such redistribution must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.moengage.core.analytics;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Nullable;
import android.support.annotation.WorkerThread;
import com.moe.pushlibrary.MoEHelper;
import com.moe.pushlibrary.models.Event;
import com.moengage.core.Logger;
import com.moengage.core.MoEConstants;
import com.moengage.core.MoECoreEvaluator;
import com.moengage.core.MoEDAO;
import com.moengage.core.MoEDispatcher;
import com.moengage.core.MoEUtils;
import com.moengage.core.RemoteConfig;
import com.moengage.core.model.TrafficSource;
import com.moengage.core.model.UserSession;
import java.util.UUID;

/**
 * @author Umang Chamaria
 * Date: 2019-05-21
 */
public class AnalyticsHelper {

  private static final String TAG = MoEConstants.MODULE_TAG + "AnalyticsHelper";

  private static AnalyticsHelper instance = null;

  private UserSession session;

  private MoECoreEvaluator evaluator;

  private SourceProcessor sourceProcessor;

  private AnalyticsHelper(Context context) {
    session = MoEDAO.getInstance(context).getLastSavedSession();
    evaluator = MoEDispatcher.getInstance(context)
        .getCoreEvaluator();
    sourceProcessor = new SourceProcessor();
  }

  public static AnalyticsHelper getInstance(Context context) {
    if (instance == null) {
      synchronized (AnalyticsHelper.class) {
        if (instance == null) instance = new AnalyticsHelper(context);
      }
    }
    return instance;
  }

  public void onEventTracked(Event event, Context context) {
    try {
      Logger.v(TAG
          + " onEventTracked() : Will update last interaction time if required. Event: "
          + event.details);
      // check if event is non-interactive
      if (!event.isInteractiveEvent) {
        Logger.v(
            TAG + " onEventTracked() : No operation required. Tracked event is non-interactive");
        return;
      }
      // user attribute tracked no action required.
      if (event.eventName.equals(MoEConstants.EVENT_ACTION_USER_ATTRIBUTE)) {
        Logger.v(TAG + " updateSession() : Need not update session info since user attribute is "
            + "tracked.");
        return;
      }
      // app in foreground no action required.
      if (MoEHelper.isAppInForeground()) {
        Logger.v(TAG + " updateSession() : App is in foreground no action required.");
        updateLastInteractionTime(MoEUtils.currentTime());
        return;
      }
      // no sessions created till now
      if (session == null) {
        Logger.v(TAG + " onEventTracked() : No previous session. Will create a new session");
        batchPreviousDataAndCreateNewSession(context, null, true);
        return;
      }
      // existing session expired create a new session.
      if (evaluator.hasSessionExpired(session.lastInteractionTime,
          RemoteConfig.getConfig().sessionInActiveTime,
          MoEUtils.currentTime())) {
        Logger.v(TAG + " onEventTracked() : Session has expired.");
        batchPreviousDataAndCreateNewSession(context, null, true);
        return;
      }
      //update last interaction
      updateLastInteractionTime(MoEUtils.currentTime());
    } catch (Exception e) {
      Logger.e(TAG + " onEventTracked() : Exception: ", e);
    }
  }

  void updateLastInteractionTime(long time) {
    if (session != null) {
      session.lastInteractionTime = time;
    }
  }

  // update session if required
  @WorkerThread public void onAppOpen(Activity activity) {
    if (session != null) {
      Logger.v(TAG + " onAppOpen() : Current Session " + session.toString());
    }
    updateUserSessionIfRequired(activity);
  }

  public void onNotificationClicked(Context context, TrafficSource source,
      boolean isAppInBackground) {
    try {
      Logger.v(TAG + " onNotificationClicked() : Source " + source);
      updateSessionIfRequired(context, source, isAppInBackground);
    } catch (Exception e) {
      Logger.e(TAG + " onNotificationClicked() : ", e);
    }
  }

  private void updateUserSessionIfRequired(Activity activity) {
    try {
      Context context = activity.getApplicationContext();
      TrafficSource currentSource = sourceProcessor.getTrafficSourceFromActivity(activity,
          RemoteConfig.getConfig().additionalSourceIdentifiers);
      updateSessionIfRequired(context, currentSource, MoEHelper.isAppInBackground());
    } catch (Exception e) {
      Logger.e(TAG + " onAppOpen() : Exception: ", e);
    }
  }

  private void updateSessionIfRequired(Context context, TrafficSource currentSource,
      boolean isAppInBackground) {
    Logger.v(TAG + " updateSessionIfRequired() : New source: " + currentSource);
    if (session == null) {
      Logger.v(TAG
          + " updateSessionIfRequired() : No saved session for user will create a new session.");
      batchPreviousDataAndCreateNewSession(context, currentSource,
          MoEHelper.isAppInBackground());
      return;
    }
    Logger.v(TAG + " updateSessionIfRequired() : Current Session: " + session);
    // previous session expired
    boolean hasSessionExpired =
        evaluator.hasSessionExpired(session.lastInteractionTime,
            RemoteConfig.getConfig().sessionInActiveTime,
            MoEUtils.currentTime());
    if (hasSessionExpired) {
      Logger.v(TAG
          + " updateSessionIfRequired() : Previous session has expired. Will create a new session"
          + ".");
      batchPreviousDataAndCreateNewSession(context, currentSource,
          MoEHelper.isAppInBackground());
      return;
    }
    // source changed
    TrafficSource savedSource = session.trafficSource;
    boolean hasSourceChanged = evaluator.hasSourceChanged(savedSource, currentSource);
    if (hasSourceChanged) {
      Logger.v(TAG + " updateSessionIfRequired() : Source changed. will create a new session");
      batchPreviousDataAndCreateNewSession(context, currentSource,
          isAppInBackground);
    }
  }

  @WorkerThread private void batchPreviousDataAndCreateNewSession(Context context,
      TrafficSource currentSource, boolean isFromBackground) {
    MoEDispatcher.getInstance(context).getBatchHelper().createAndSaveBatches(context, session);
    MoEDispatcher.getInstance(context).sendInteractionData();
    createAndPersistNewSession(context, currentSource, isFromBackground);
  }

  private UserSession createAndPersistNewSession(Context context,
      TrafficSource currentSource, boolean isFromBackground) {
    session = createNewSession(currentSource, isFromBackground);
    Logger.v(TAG + " createAndPersistNewSession() : New session: " + session.toString());
    persistUserSession(context, session);
    return session;
  }

  private UserSession createNewSession(@Nullable TrafficSource currentSource,
      boolean isFromBackground) {
    long currentTime = MoEUtils.currentTime();
    return new UserSession(
        UUID.randomUUID().toString(),
        MoEUtils.getTimeInISO(currentTime),
        currentSource,
        currentTime,
        isFromBackground
    );
  }

  private void persistUserSession(Context context, UserSession userSession) {
    MoEDAO.getInstance(context).saveUserSession(userSession);
  }

  public void onAppClose(Context context) {
    updateLastInteractionTime(MoEUtils.currentTime());
    persistUserSession(context, session);
  }

  @WorkerThread public void onLogout(Context context) {
    createAndPersistNewSession(context, null, MoEHelper.isAppInBackground());
  }

  @Nullable public UserSession getSession() {
    return session;
  }
}

==> moengagesdk-4.2.00-sources.jar: com/moengage/core/analytics/AnalyticsHelper.java <==
/*
 * Copyright (c) 2014-2020 MoEngage Inc.
 *
 * All rights reserved.
 *
 *  Use of source code or binaries contained within MoEngage SDK is permitted only to enable use of the MoEngage platform by customers of MoEngage.
 *  Modification of source code and inclusion in mobile apps is explicitly allowed provided that all other conditions are met.
 *  Neither the name of MoEngage nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *  Redistribution of source code or binaries is disallowed except with specific prior written permission. Any such redistribution must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.moengage.core.analytics;

import android.app.Activity;
import android.content.Context;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import com.moe.pushlibrary.MoEHelper;
import com.moe.pushlibrary.models.Event;
import com.moengage.core.Logger;
import com.moengage.core.MoEConstants;
import com.moengage.core.MoECoreEvaluator;
import com.moengage.core.MoEDAO;
import com.moengage.core.MoEDispatcher;
import com.moengage.core.MoEUtils;
import com.moengage.core.RemoteConfig;
import com.moengage.core.model.TrafficSource;
import com.moengage.core.model.UserSession;
import java.util.UUID;

/**
 * @author Umang Chamaria
 * Date: 2019-05-21
 */
public class AnalyticsHelper {

  private static final String TAG = MoEConstants.MODULE_TAG + "AnalyticsHelper";

  private static AnalyticsHelper instance = null;

  private UserSession session;

  private MoECoreEvaluator evaluator;

  private SourceProcessor sourceProcessor;

  private AnalyticsHelper(Context context) {
    session = MoEDAO.getInstance(context).getLastSavedSession();
    evaluator = new MoECoreEvaluator();
    sourceProcessor = new SourceProcessor();
  }

  public static AnalyticsHelper getInstance(Context context) {
    if (instance == null) {
      synchronized (AnalyticsHelper.class) {
        if (instance == null) instance = new AnalyticsHelper(context);
      }
    }
    return instance;
  }

  public void onEventTracked(Event event, Context context) {
    try {
      Logger.v(TAG
          + " onEventTracked() : Will update last interaction time if required. Event: "
          + event.details);
      // check if event is non-interactive
      if (!event.isInteractiveEvent) {
        Logger.v(
            TAG + " onEventTracked() : No operation required. Tracked event is non-interactive");
        return;
      }
      // user attribute tracked no action required.
      if (event.eventName.equals(MoEConstants.EVENT_ACTION_USER_ATTRIBUTE)) {
        Logger.v(TAG + " updateSession() : Need not update session info since user attribute is "
            + "tracked.");
        return;
      }
      // app in foreground no action required.
      if (MoEHelper.isAppInForeground()) {
        Logger.v(TAG + " updateSession() : App is in foreground no action required.");
        updateLastInteractionTime(MoEUtils.currentMillis());
        return;
      }
      // no sessions created till now
      if (session == null) {
        Logger.v(TAG + " onEventTracked() : No previous session. Will create a new session");
        batchPreviousDataAndCreateNewSession(context, null, true);
        return;
      }
      // existing session expired create a new session.
      if (evaluator.hasSessionExpired(session.lastInteractionTime,
          RemoteConfig.getConfig().sessionInActiveTime,
          MoEUtils.currentMillis())) {
        Logger.v(TAG + " onEventTracked() : Session has expired.");
        batchPreviousDataAndCreateNewSession(context, null, true);
        return;
      }
      //update last interaction
      updateLastInteractionTime(MoEUtils.currentMillis());
    } catch (Exception e) {
      Logger.e(TAG + " onEventTracked() : Exception: ", e);
    }
  }

  void updateLastInteractionTime(long time) {
    if (session != null) {
      session.lastInteractionTime = time;
    }
  }

  // update session if required
  @WorkerThread public void onAppOpen(Activity activity) {
    if (session != null) {
      Logger.v(TAG + " onAppOpen() : Current Session " + session.toString());
    }
    updateUserSessionIfRequired(activity);
  }

  public void onNotificationClicked(Context context, TrafficSource source,
      boolean isAppInBackground) {
    try {
      Logger.v(TAG + " onNotificationClicked() : Source " + source);
      updateSessionIfRequired(context, source, isAppInBackground);
    } catch (Exception e) {
      Logger.e(TAG + " onNotificationClicked() : ", e);
    }
  }

  private void updateUserSessionIfRequired(Activity activity) {
    try {
      Context context = activity.getApplicationContext();
      TrafficSource currentSource = sourceProcessor.getTrafficSourceFromActivity(activity,
          RemoteConfig.getConfig().additionalSourceIdentifiers);
      updateSessionIfRequired(context, currentSource, MoEHelper.isAppInBackground());
    } catch (Exception e) {
      Logger.e(TAG + " onAppOpen() : Exception: ", e);
    }
  }

  private void updateSessionIfRequired(Context context, TrafficSource currentSource,
      boolean isAppInBackground) {
    Logger.v(TAG + " updateSessionIfRequired() : New source: " + currentSource);
    if (session == null) {
      Logger.v(TAG
          + " updateSessionIfRequired() : No saved session for user will create a new session.");
      batchPreviousDataAndCreateNewSession(context, currentSource,
          MoEHelper.isAppInBackground());
      return;
    }
    Logger.v(TAG + " updateSessionIfRequired() : Current Session: " + session);
    // previous session expired
    boolean hasSessionExpired =
        evaluator.hasSessionExpired(session.lastInteractionTime,
            RemoteConfig.getConfig().sessionInActiveTime,
            MoEUtils.currentMillis());
    if (hasSessionExpired) {
      Logger.v(TAG
          + " updateSessionIfRequired() : Previous session has expired. Will create a new session"
          + ".");
      batchPreviousDataAndCreateNewSession(context, currentSource,
          MoEHelper.isAppInBackground());
      return;
    }
    // source changed
    TrafficSource savedSource = session.trafficSource;
    boolean hasSourceChanged = evaluator.hasSourceChanged(savedSource, currentSource);
    if (hasSourceChanged) {
      Logger.v(TAG + " updateSessionIfRequired() : Source changed. will create a new session");
      batchPreviousDataAndCreateNewSession(context, currentSource,
          isAppInBackground);
    }
  }

  @WorkerThread private void batchPreviousDataAndCreateNewSession(Context context,
      TrafficSource currentSource, boolean isFromBackground) {
    MoEDispatcher.getInstance(context).getBatchHelper().createAndSaveBatches(context, session);
    MoEDispatcher.getInstance(context).sendInteractionData();
    createAndPersistNewSession(context, currentSource, isFromBackground);
  }

  private UserSession createAndPersistNewSession(Context context,
      TrafficSource currentSource, boolean isFromBackground) {
    session = createNewSession(currentSource, isFromBackground);
    Logger.v(TAG + " createAndPersistNewSession() : New session: " + session.toString());
    persistUserSession(context, session);
    return session;
  }

  private UserSession createNewSession(@Nullable TrafficSource currentSource,
      boolean isFromBackground) {
    long currentTime = MoEUtils.currentMillis();
    return new UserSession(
        UUID.randomUUID().toString(),
        MoEUtils.getTimeInISO(currentTime),
        currentSource,
        currentTime,
        isFromBackground
    );
  }

  private void persistUserSession(Context context, UserSession userSession) {
    MoEDAO.getInstance(context).saveUserSession(userSession);
  }

  public void onAppClose(Context context) {
    updateLastInteractionTime(MoEUtils.currentMillis());
    persistUserSession(context, session);
  }

  @WorkerThread public void onLogout(Context context) {
    createAndPersistNewSession(context, null, MoEHelper.isAppInBackground());
  }

  @Nullable public UserSession getSession() {
    return session;
  }
}
